package org.greenstone.gsdl3.service;

// Greenstone classes
import org.greenstone.gdbm.*;
import org.greenstone.gsdl3.util.*;

// XML classes
import org.w3c.dom.Element; 
import org.w3c.dom.Document;
import org.w3c.dom.NodeList; 

//Java classes
import java.util.ArrayList;
import java.util.HashMap;
import java.io.File;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URLConnection;
import java.net.URL;
import java.net.Authenticator;
import java.net.MalformedURLException;

/**
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 9273 $
 * Modified by <a href="mailto:chi@cs.waikato.ac.nz">Chi-Yu Huang</a>
 */

public class IViaSearch
    extends AbstractSearch {
    
    // have standard gs param names for hits per page, and start page
    // these need to be mapped to iVia params
    protected static final String IM_HITS_PARAM = "no_of_records_per_page";
    protected static final String IM_START_PAGE_PARAM = "start_page_no";

    protected String ivia_server_url = null;

    public IViaSearch()
    {
    }
    
    //Configure IViaSearch Service
    public boolean configure(Element info, Element extra_info) 
    { 
	Element server_elem = (Element)GSXML.getChildByTagName(info, "iViaServer");
	if (server_elem == null) {
	    System.err.println("IViaSearch.configure error: no iViaServer element found");
	    return false;
	}
	ivia_server_url = server_elem.getAttribute("url");
	if (ivia_server_url.equals("")) {
	    System.err.println("IViaSearch.configure error: no url for the iViaServer element");
	    return false;
	}
	does_paging = true;
	does_multi_index_search = true;
	return super.configure(info, extra_info);
    }
    
    /** Process a text query - implemented by concrete subclasses */
    protected Element processTextQuery(Element request) {
	// Create a new (empty) result message
	Element result = this.doc.createElement(GSXML.RESPONSE_ELEM);
	result.setAttribute(GSXML.FROM_ATT, TEXT_QUERY_SERVICE);
	result.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	Element doc_node_list = this.doc.createElement(GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	result.appendChild(doc_node_list);
	Element param_list = (Element) GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (param_list == null) {
	    System.err.println("IViaSearch Error:: TextQuery request had no paramList.");
	    return result;  // Return the empty result
	}

	// Process the request parameters
	HashMap params = GSXML.extractParams(param_list, false);

	// Make sure a query has been specified
	String query = (String) params.get(QUERY_PARAM);
	if (query == null || query.equals("")) {
	    return result;  // Return the empty result
	}

	// tidy whitespace
	query = query.replaceAll("\\s+", "+");
	String url_string = ivia_server_url+"/cgi-bin/canned_search?theme=gsdl3&query="+query; 

	// check for fields
	String fields = (String) params.get(INDEX_PARAM);
	if (fields != null && !fields.equals("")) {
	    url_string += "&fields="+fields;
	}

	//check for hits per page
	String hits_per_page = (String) params.get(HITS_PER_PAGE_PARAM);
	if (hits_per_page != null && !hits_per_page.equals("")) {
	    url_string += "&"+IM_HITS_PARAM+"="+hits_per_page;
	}

	// check for start page
	String start_page = (String) params.get(START_PAGE_PARAM);
	if (start_page != null && !start_page.equals("")) {
	    url_string += "&"+IM_START_PAGE_PARAM+"="+start_page;
	}
	String results_num = null;
	String doc_ids = null;
	try {
	    ///system.err.println("IViaSearch, sending "+url_string);
	    BufferedReader reader = makeConnection(url_string);
	    results_num = reader.readLine();
	    doc_ids = reader.readLine();
	} catch (Exception e) {
	    System.err.println("IViaSearch.TextQuery Error: exception happened during query");
	    e.printStackTrace();
	    return result;
	}
	    
	if (results_num.startsWith("Resources: ")) {
	    results_num = results_num.substring(11);
	} else {
	    System.err.println("IViaSearch.TextQuery Error: badly formatted results line: "+results_num);
	    return result;
	}
	if (doc_ids.startsWith("Ids: ")) {
	    doc_ids = doc_ids.substring(5).trim();
	} else {
	    System.err.println("IViaSearch.TextQuery Error: badly formatted docs line: "+doc_ids);
	    return result;
	}
	
	// get the num docs and add to a metadata list
	Element metadata_list = this.doc.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER); 
	result.appendChild(metadata_list);
	
	// Add a metadata element specifying the number of matching documents
	long numdocs = Long.parseLong(results_num);
	GSXML.addMetadata(this.doc, metadata_list, "numDocsMatched", ""+numdocs);	
	String [] ids = doc_ids.split(" ");
	
	for (int d=0; d<ids.length; d++) {
	    Element doc_node = this.doc.createElement(GSXML.DOC_NODE_ELEM);
	    doc_node.setAttribute(GSXML.NODE_ID_ATT, ids[d]);
	    doc_node_list.appendChild(doc_node);
	}
	return result;
    }
     
    protected BufferedReader makeConnection(String url_string) {
	BufferedReader reader = null;
	try {
	    URL url = new URL(url_string);
	    HttpURLConnection connection = (HttpURLConnection)url.openConnection();
	    InputStream input = connection.getInputStream();
	    reader = new BufferedReader(new InputStreamReader(input));  
	} catch (java.net.MalformedURLException e) {
	    System.err.println("IViaSearch Error: Malformed URL: "+url_string);
	} catch (java.io.IOException e) {
	    System.err.println("IViaSearch Error: An error occurred during IO to url "+url_string);
	}
	return reader;
    }

    /**
       An IVia server has a fixed list of fields to search (I think) so we can hard code them here rather than reading them in from a config file
    */
    protected void getIndexData(ArrayList index_ids, ArrayList index_names,String lang){
	index_ids.add("kw");
	index_ids.add("au");	
	index_ids.add("su");	
	index_ids.add("ti");
	index_ids.add("de");
	index_ids.add("fu");
	index_names.add(getTextString("param."+INDEX_PARAM+".kw", lang)); 
	index_names.add(getTextString("param."+INDEX_PARAM+".au", lang));
	index_names.add(getTextString("param."+INDEX_PARAM+".su", lang));
	index_names.add(getTextString("param."+INDEX_PARAM+".ti", lang));
	index_names.add(getTextString("param."+INDEX_PARAM+".de", lang));
	index_names.add(getTextString("param."+INDEX_PARAM+".fu", lang));
    }
    
}
